/*
 * Main.c
 *
 * Created: 26.02.2013 14:04:47
 *  Author: Stefan Rttimann
 */ 


// ************************************************************************
// * Include's fr main
// ************************************************************************	
	
	#include <avr/io.h>
	#include <avr/sleep.h>
	#include <avr/wdt.h>
	#include "delay.h"
	#include "Display.h"
	#include "LED.h"
	#include "Switch_Dash.h"
	#include "Switch_Lenk.h"

	//for CAN
	#include "can_drv.h"
	#include "can_lib.h"
	#include "compiler.h"
	#include "at90can_drv.h"
	#include "config.h"


	void Init_System();
	void Init_CAN_Data();

// ************************************************************************
// * Globale Variablen um mit ISR zu Komunizieren
// ************************************************************************
	volatile uint16_t SystemCounter = 1;

// ************************************************************************

// ************************************************************************
// * CAN Node
// ************************************************************************	
	#define RX_TAG 0x602													//RX ID-CAN
	#define TX_TAG 0x601													//TX ID-CAN

	#define RX_DATA_LENGTH	4												//Anzahl Bytes zum Empfangen
	#define TX_DATA_LENGTH	4												//Anzahl Bytes zum Senden

	static st_cmd_t TX_Daten, RX_Daten;
	static U8 CAN_TX_Data[TX_DATA_LENGTH], CAN_RX_Data[RX_DATA_LENGTH];

// ************************************************************************
// * Main
// * 1) Initialisiert MC
// * 2) Arbeitet SW ab in endlos Loop mit T = 200ms.
// ************************************************************************

int main (void)
{
		
	// Variablen Switch
	volatile uint8_t reserve=0, tS=0, tC=0, tV=0, recup=0, kobi=0;				//Switch Dashboard
	volatile uint8_t flappy1=0, flappy2=0, flappy1_vor = 0, flappy2_vor = 0, 
						plus=0, minus=0, plusvor=0, minusvor=0;					//Switch Lenkrad
						
	
	// Variablen LED
	uint8_t LEDGr=0, LEDRd=0;													//LEDs Gruen/Rot

	//variablen CAN / Buffer
	uint8_t CAN_counter = 0;													//CAN Counter Variabel
	uint8_t LEDAllg = 0;														//Fehler Allgemein
	uint8_t LEDBMS = 0;															//BMS Error
	uint8_t LEDMotor = 0;														//Motor temp zu hoch
	uint8_t LEDAkku = 0;														//Akku temp zu hoch
	uint8_t LEDHV = 0;															//HV Spannung low
	uint8_t LEDLV = 0;															//LV Spannung low
	uint8_t LEDGFD = 0;															//GFD Error
	uint8_t LEDRes = 0;															//Reserve
	
	uint8_t Wert1Disp = 0;														//1. Wert fr Display
	uint8_t Wert2Disp = 0;														//2. Wert fr Display
	uint8_t FehlCode = 0;														//Fehler Code (Allg. Fehler)
	
	uint8_t DispMod = 0;														//Anzeigemodus des Displays
	uint8_t MotPowFr = 0;														//Einstellbare Motor Power
	uint8_t MotPowRe = 0;														//Einstellbare Motor Power
	uint8_t TrCont = 0;															//Einstellbare Traction Control
	uint8_t TorVec = 0;															//Einstellbares Torque Vectoring
	uint8_t Verstel = 0;														//Verstellbare Werte in zwei Bytes (Motor/TC/TC)
	
	Init_System();

	// Endlos loop fr Abarbeitung
	while(1){	
		
		//************************************
		// Einlesen von Switch
		//************************************	

		reserve = getSwitchStateDash(SW_Reserve);								//Einlesen Reserve Schalter
		tS = getSwitchStateDash(SW_TS_St);										//Einlesen TS Start Schalter
		tC = getSwitchStateDash(SW_TC);											//Einlesen Traction Control Schalter
		tV = getSwitchStateDash(SW_TV);											//Einlesen Torque Vectoring Schalter
		recup = getSwitchStateDash(SW_Recup);									//Einlesen Recuperation Schalter
		kobi = getSwitchStateDash(SW_Kobi);										//Einlesen Kobayashi Schalter
		
		flappy1 = getSwitchStateLenk(SW_FL1);									//Einlesen Flappy1 Schalter
		flappy2 = getSwitchStateLenk(SW_FL2);									//Einlesen Flappy2 Schalter
		minus = getSwitchStateLenk(SW_MINUS);									//Einlesen Minus Schalter
		plus = getSwitchStateLenk(SW_PLUS);										//Einlesen Plus Schalter
		
		//************************************

		//************************************
		// Einlesen der Daten von CAN
		//************************************
		
		CAN_counter++;
		if(can_get_status(&RX_Daten) == CAN_STATUS_COMPLETED){
			CAN_counter = 0;
			cli();
			LEDAllg = ((CAN_RX_Data[0] & 0b00000001));						//Datenbit 0 aus CANByte 0 zuweisen
			LEDBMS = ((CAN_RX_Data[0] & 0b00000010) >> 1);					//Datenbit 1 aus CANByte 0 zuweisen
			LEDMotor = ((CAN_RX_Data[0] & 0b00000100) >> 2);				//Datenbit 2 aus CANByte 0 zuweisen
			LEDAkku = ((CAN_RX_Data[0] & 0b00001000) >> 3);					//Datenbit 3 aus CANByte 0 zuweisen
			LEDHV = ((CAN_RX_Data[0] & 0b00010000) >> 4);					//Datenbit 4 aus CANByte 0 zuweisen
			LEDLV = ((CAN_RX_Data[0] & 0b00100000) >> 5);					//Datenbit 5 aus CANByte 0 zuweisen
			LEDGFD = ((CAN_RX_Data[0] & 0b01000000) >> 6);					//Datenbit 6 aus CANByte 0 zuweisen
			LEDRes = ((CAN_RX_Data[0] & 0b10000000) >> 7);					//Datenbit 7 aus CANByte 0 zuweisen
			
			Wert1Disp = CAN_RX_Data[1];										//CANByte 1 zuweisen
			Wert2Disp = CAN_RX_Data[2];										//CANByte 2 zuweisen
			FehlCode = CAN_RX_Data[3];										//CANByte 3 zuweisen
			
			RX_Daten.cmd = CMD_RX_DATA;
			can_cmd(&RX_Daten);												//Bereit zum Empfangen der Daten

			sei();
		}
		
		if(CAN_counter >= 5){												//CAN-Komm. unterbrochen nach 5mal kein Empfang auf CAN
			CAN_counter = 6;
			
			FehlCode = 0b11111111;											//Fehlercode fr CAN Unterbruch
					
			RX_Daten.cmd = CMD_ABORT;
			can_cmd(&RX_Daten);												//Abbruch Datenempfang
			while(can_get_status(&RX_Daten) == CAN_STATUS_NOT_COMPLETED);
			RX_Daten.cmd = CMD_RX_DATA;
			can_cmd(&RX_Daten);												//Abbruch Datenempfang			
		}
		//************************************

		//************************************
		// Ausgabe von Daten auf LED
		//************************************
		
		LEDGr = 0;
		
		//Einzelne LED Status in ein Statusbyte zusammentragen
		LEDGr |= (LEDAllg << 7);
		LEDGr |= (LEDBMS);
		LEDGr |= (LEDMotor << 2);
		LEDGr |= (LEDAkku << 3);
		LEDGr |= (LEDHV << 5);
		LEDGr |= (LEDLV << 4);
		LEDGr |= (LEDGFD << 6);
		
		SetLED_Green(LEDGr);												//Grne LED einschalten
		
		LEDRd = ~(LEDGr);													//Alle LED rot, welche nicht grn sind
		SetLED_Red(LEDRd);													//Rote LED einschalten
		
		//************************************
		
		//************************************
		// Ausgabe von Daten auf Display
		//************************************
		
		if ((plus >= 1) && (plusvor == 0))
		{
			plusvor = plus;
			plus = 1;
		}
		else
		{
			plusvor = plus;
			plus = 0;
		}
		if ((minus >= 1) && (minusvor == 0))
		{
			minusvor = minus;
			minus = 1;
		}
		else
		{
			minusvor = minus;
			minus = 0;
		}

		
		if ((flappy1 >= 1) && (flappy1_vor == 0))
		{
			if (DispMod < 11)
			{
				DispMod += 1;
			}
			else
			{
				DispMod = 0;
			}
		}
		else if ((flappy2 >= 1) && (flappy2_vor == 0))
		{
			if (DispMod > 0)
			{
				DispMod -= 1;
			} 
			else
			{
				DispMod = 11;
			}
		}
		flappy1_vor = flappy1;
		flappy2_vor = flappy2;
		
		Verstel = ShowDisplay(DispMod, Wert1Disp, Wert2Disp, plus, minus, FehlCode);
		
		if (DispMod == 7)
		{
			MotPowFr = Verstel;
		}
		if (DispMod == 8)
		{
			MotPowRe = Verstel;
		}
		if (DispMod == 9)
		{
			TrCont = Verstel;
		}
		if (DispMod == 10)
		{
			TorVec = Verstel;
		}
		
		//************************************


		//************************************
		// Senden der Daten an CAN
		//************************************
		
		CAN_TX_Data[0] = 0;
		CAN_TX_Data[1] = 0;
		CAN_TX_Data[2] = 0;
		CAN_TX_Data[3] = 0;
		
		CAN_TX_Data[0] = CAN_TX_Data[0] | (tS >> 3);
		CAN_TX_Data[0] = CAN_TX_Data[0] | (tC >> 3);
		CAN_TX_Data[0] = CAN_TX_Data[0] | (tV >> 3);
		CAN_TX_Data[0] = CAN_TX_Data[0] | (recup >> 3);
		CAN_TX_Data[0] = CAN_TX_Data[0] | (kobi >> 3);
		CAN_TX_Data[0] = CAN_TX_Data[0] | (reserve << 3);
		
		CAN_TX_Data[1] = CAN_TX_Data[1] | (DispMod);
		CAN_TX_Data[2] = CAN_TX_Data[2] | (TrCont);
		CAN_TX_Data[2] = CAN_TX_Data[2] | (TorVec << 4);
		CAN_TX_Data[3] = CAN_TX_Data[3] | (MotPowFr);
		CAN_TX_Data[3] = CAN_TX_Data[3] | (MotPowRe << 4);
			
		TX_Daten.cmd = CMD_TX_DATA;
		can_cmd(&TX_Daten);
		
		//Senden UND Warten bis gesendet
		while(can_get_status(&TX_Daten) == CAN_STATUS_NOT_COMPLETED); 
		//************************************

		//************************************
		// Warte bis 200ms over
		//************************************
		while(SystemCounter);												//Warte bis 200ms vorbei (Systemcounter = 0)
		wdt_reset();														//Programm ok
		SystemCounter = 1;													//SystemCounter zurcksetzen
		//************************************

		
	}

	return 0;
}

// ************************************************************************
// * ISR fr Timer 1
// ************************************************************************


EMPTY_INTERRUPT(__vector_default);

ISR(TIMER1_COMPA_vect)
{
	SystemCounter = 0;
}

// ************************************************************************

// ************************************************************************
// * Initialisiert MC
// ************************************************************************

void Init_System (void)
{
	cli();																	//All interrupts disable
	InitTimer1();															//Timer fr 200ms Takt Initialisieren
	InitDisplay();															//Init Display
	InitLED();																//Init LED's
	InitSwitchDash();														//Init Schaltereingnge Dashboard
	InitSwitchLenk();														//Init Schaltereingnge Lenkrad
	can_init(0);															//Init CAN -> siehe "config.h"
	Init_CAN_Data();
	can_cmd(&RX_Daten);														//Bereit zum Empfangen der Daten
	Timer1_On();															//Timer starten
	sei();																	//General interrupt enable
	
}

void Init_CAN_Data()
{
		TX_Daten.pt_data = CAN_TX_Data;										//Zuweisung der Daten (Array 8 Byte)
		TX_Daten.cmd = CMD_TX_DATA;											//Befehl fr CAN bus
		TX_Daten.ctrl.ide = 0;												//0 = 11 Bit Adresse, 1 = 29 Bit Adresse
		TX_Daten.ctrl.rtr = 0;												//Remote Frame = 1; TX = 0
		TX_Daten.id.std = TX_TAG;											//ID TAG der CAN-Message
		TX_Daten.dlc = TX_DATA_LENGTH;										//Anz. zu sendende Bytes (max. 8)

		RX_Daten.pt_data = CAN_RX_Data;										//Zuweisung der Daten (Array 8 Byte)
		RX_Daten.cmd = CMD_RX_DATA;											//Befehl fr CAN bus
		RX_Daten.ctrl.ide = 0;												//0 = 11 Bit Adresse, 1 = 29 Bit Adresse
		RX_Daten.ctrl.rtr = 0;												//Remote Frame = 1; TX = 0
		RX_Daten.id.std = RX_TAG;											//ID TAG der CAN-Message
		RX_Daten.dlc = RX_DATA_LENGTH;										//Anz. zu sendende Bytes (max. 8)
		RX_Daten.id_mask = 0x7FF;											//Wird nicht gebraucht
}
